@@ -1,18 +1,17 @@
 // TODO: Block comments.
-use nom::{
-    branch::alt,
-    character::complete::char,
-    error::{context, Error, ErrorKind, ParseError},
-    multi::many0,
-    sequence::{delimited, pair, preceded, terminated},
-    IResult, InputTake,
-};
-
 use crate::Sexp;
 
-type Res<T, U> = IResult<T, U, Error<T>>;
+#[derive(Clone, Debug, PartialEq, Eq)]
+pub enum Error {
+    UnexpectedCharInString(u8),
+    UnexpectedEofInString,
+    UnexpectedEof,
+    EmptyAtom,
+}
 
-fn space_or_comments(input: &[u8]) -> Res<&[u8], &[u8]> {
+type Res<'a, T> = Result<(&'a [u8], T), Error>;
+
+fn space_or_comments(input: &[u8]) -> Res<()> {
     let mut index = 0;
     while index < input.len() {
         match input[index] {
@@ -22,46 +21,36 @@ fn space_or_comments(input: &[u8]) -> Res<&[u8], &[u8]> {
                     index += 1
                 }
             }
-            _ => return Ok(input.take_split(index)),
+            _ => return Ok((&input[index..], ())),
         }
     }
-    Ok(input.take_split(input.len()))
+    Ok((&[], ()))
 }
 
-fn unquoted_string_(input: &[u8]) -> Res<&[u8], &[u8]> {
-    // Most errors below are handled with Failure rather than error
-    // as these cannot be recovered from.
+fn unquoted_string_(input: &[u8]) -> Res<&[u8]> {
     for (index, &c) in input.iter().enumerate() {
         match c {
             b';' | b'(' | b')' | b'"' | b' ' | b'\t' | b'\r' | b'\n' => {
-                return Ok(input.take_split(index));
+                let (str, remaining) = input.split_at(index);
+                return Ok((remaining, str));
             }
             b'#' if index > 0 && input[index - 1] == b'|' => {
-                return Err(nom::Err::Failure(Error::from_error_kind(
-                    input,
-                    ErrorKind::Not,
-                )));
+                return Err(Error::UnexpectedCharInString(b'|'))
             }
             b'|' if index > 0 && input[index - 1] == b'#' => {
-                return Err(nom::Err::Failure(Error::from_error_kind(
-                    input,
-                    ErrorKind::Not,
-                )));
+                return Err(Error::UnexpectedCharInString(b'#'))
             }
             _ => {}
         }
     }
-    Ok(input.take_split(input.len()))
+    Ok((&[], input))
 }
 
-fn unquoted_string(input: &[u8]) -> Res<&[u8], Vec<u8>> {
+fn unquoted_string(input: &[u8]) -> Res<Vec<u8>> {
     match unquoted_string_(input) {
         Ok((next_input, atom)) => {
             if atom.is_empty() {
-                Err(nom::Err::Error(Error::from_error_kind(
-                    input,
-                    ErrorKind::NonEmpty,
-                )))
+                Err(Error::EmptyAtom)
             } else {
                 Ok((next_input, atom.to_vec()))
             }
@@ -114,23 +103,20 @@ fn two_hex_digits(input: &[u8], index: usize) -> Option<u8> {
 }
 
 // Maybe this should be rewritten using combinators?
-fn quoted_string(input: &[u8]) -> Res<&[u8], Vec<u8>> {
+fn quoted_string(input: &[u8]) -> Res<Vec<u8>> {
     let mut buffer: Vec<u8> = Vec::new();
     let mut index = 0;
     while index < input.len() {
         match input[index] {
             b'"' => {
-                let (tail, _) = input.take_split(index);
-                return Ok((tail, buffer));
+                let (_, remaining) = input.split_at(index);
+                return Ok((remaining, buffer));
             }
             b'\\' => {
                 index += 1;
                 if index == input.len() {
                     // Unexpected eof
-                    return Err(nom::Err::Failure(Error::from_error_kind(
-                        input,
-                        ErrorKind::Eof,
-                    )));
+                    return Err(Error::UnexpectedEofInString);
                 }
                 match input[index] {
                     b'\n' => {
@@ -182,46 +168,67 @@ fn quoted_string(input: &[u8]) -> Res<&[u8], Vec<u8>> {
         };
         index += 1;
     }
-    Err(nom::Err::Failure(Error::from_error_kind(
-        input,
-        ErrorKind::Eof,
-    )))
+    Err(Error::UnexpectedEofInString)
 }
 
-fn atom(input: &[u8]) -> Res<&[u8], Sexp> {
-    context(
-        "atom",
-        alt((
-            unquoted_string,
-            delimited(char('"'), quoted_string, char('"')),
-        )),
-    )(input)
-    .map(|(next_input, atom)| (next_input, Sexp::Atom(atom)))
+fn first_char_is(c: u8, input: &[u8]) -> bool {
+    input.first().map(|x| *x == c).unwrap_or(false)
 }
 
-fn sexp_in_list(input: &[u8]) -> Res<&[u8], Sexp> {
-    context(
-        "sexp-in-list",
-        delimited(
-            pair(char('('), space_or_comments),
-            many0(sexp_no_leading_blank),
-            char(')'),
-        ),
-    )(input)
-    .map(|(next_input, res)| (next_input, Sexp::List(res)))
+fn char(c: u8, input: &[u8]) -> Res<()> {
+    if first_char_is(c, input) {
+        Ok((&input[1..], ()))
+    } else {
+        Err(Error::UnexpectedEof)
+    }
+}
+
+fn atom(input: &[u8]) -> Res<Sexp> {
+    let (next_input, atom) = if first_char_is(b'"', input) {
+        let (input, ()) = char(b'"', input)?;
+        let (input, atom) = quoted_string(input)?;
+        let (input, ()) = char(b'"', input)?;
+        (input, atom)
+    } else {
+        unquoted_string(input)?
+    };
+    Ok((next_input, Sexp::Atom(atom)))
+}
+
+fn sexp_in_list(input: &[u8]) -> Res<Sexp> {
+    let (input, ()) = char(b'(', input)?;
+    let (input, ()) = space_or_comments(input)?;
+    let mut input = input;
+    let mut res = vec![];
+    while let Ok((next_input, sexp)) = sexp_no_leading_blank(input) {
+        input = next_input;
+        res.push(sexp)
+    }
+    let (input, ()) = char(b')', input)?;
+    Ok((input, Sexp::List(res)))
 }
 
 // This is used to encode a list separated by spaces as the
 // separated_list combinator does not seem to handle separators that
 // can be empty.
-fn sexp_no_leading_blank(input: &[u8]) -> Res<&[u8], Sexp> {
-    terminated(alt((atom, sexp_in_list)), space_or_comments)(input)
+fn sexp_no_leading_blank(input: &[u8]) -> Res<Sexp> {
+    if first_char_is(b'(', input) {
+        let (input, sexp) = sexp_in_list(input)?;
+        let (input, ()) = space_or_comments(input)?;
+        Ok((input, sexp))
+    } else {
+        let (input, sexp) = atom(input)?;
+        let (input, ()) = space_or_comments(input)?;
+        Ok((input, sexp))
+    }
 }
 
 /// Deserialize a Sexp from bytes, returning both the sexp and the remaining
 /// bytes.
-pub fn from_slice_allow_remaining<T: AsRef<[u8]> + ?Sized>(input: &T) -> Res<&[u8], Sexp> {
-    preceded(space_or_comments, sexp_no_leading_blank)(input.as_ref())
+pub fn from_slice_allow_remaining<T: AsRef<[u8]> + ?Sized>(input: &T) -> Res<Sexp> {
+    let input = input.as_ref();
+    let (input, ()) = space_or_comments(input)?;
+    sexp_no_leading_blank(input)
 }
 
 /// Deserialize a Sexp from bytes. This fails if there are remaining bytes.
@@ -240,16 +247,13 @@ pub fn from_slice_allow_remaining<T: AsRef<[u8]> + ?Sized>(input: &T) -> Res<&[u
 ///
 /// This deserialization can fail if the bytes do not follow the expected
 /// sexp format.
-pub fn from_slice<T: AsRef<[u8]> + ?Sized>(input: &T) -> Result<Sexp, nom::Err<Error<&[u8]>>> {
+pub fn from_slice<T: AsRef<[u8]> + ?Sized>(input: &T) -> Result<Sexp, Error> {
     let input = input.as_ref();
     let (remaining, sexp) = from_slice_allow_remaining(input)?;
     if remaining.is_empty() {
         Ok(sexp)
     } else {
-        Err(nom::Err::Failure(Error::from_error_kind(
-            remaining,
-            ErrorKind::Eof,
-        )))
+        Err(Error::UnexpectedEof)
     }
 }
 
@@ -268,18 +272,19 @@ pub fn from_slice<T: AsRef<[u8]> + ?Sized>(input: &T) -> Result<Sexp, nom::Err<E
 /// This deserialization can fail if the bytes do not follow the expected
 /// sexp format.
 
-pub fn from_slice_multi<T: AsRef<[u8]> + ?Sized>(
-    input: &T,
-) -> Result<Vec<Sexp>, nom::Err<Error<&[u8]>>> {
+pub fn from_slice_multi<T: AsRef<[u8]> + ?Sized>(input: &T) -> Result<Vec<Sexp>, Error> {
     let input = input.as_ref();
-    let (remaining, sexps) = preceded(space_or_comments, many0(sexp_no_leading_blank))(input)?;
-    if remaining.is_empty() {
+    let (input, ()) = space_or_comments(input)?;
+    let mut input = input;
+    let mut sexps = vec![];
+    while let Ok((next_input, sexp)) = sexp_no_leading_blank(input) {
+        input = next_input;
+        sexps.push(sexp)
+    }
+    if input.is_empty() {
         Ok(sexps)
     } else {
-        Err(nom::Err::Failure(Error::from_error_kind(
-            remaining,
-            ErrorKind::Eof,
-        )))
+        Err(Error::UnexpectedEof)
     }
 }
 
@@ -300,18 +305,11 @@ mod tests {
         assert_eq!(from_slice(b"( ATOM)"), Ok(Sexp::List(vec![atom(b"ATOM")])));
         assert_eq!(
             from_slice(b" ( \"foo bar\"   baz \"x\\\"\") "),
-            Ok(Sexp::List(vec![
-                atom(b"foo bar"),
-                atom(b"baz"),
-                atom(b"x\""),
-            ]))
+            Ok(Sexp::List(vec![atom(b"foo bar"), atom(b"baz"), atom(b"x\""),]))
         );
         assert_eq!(from_slice(b"\"\""), Ok(atom(b"")));
         assert_eq!(from_slice(b"\"\\000A\\123\""), Ok(atom(b"\0A\x7B")));
-        assert_eq!(
-            from_slice(b"\"\\000A\\x7B\\x99\\x9Z\""),
-            Ok(atom(b"\0A\x7B\x99\\x9Z"))
-        );
+        assert_eq!(from_slice(b"\"\\000A\\x7B\\x99\\x9Z\""), Ok(atom(b"\0A\x7B\x99\\x9Z")));
         assert_eq!(from_slice(b"( )"), Ok(list(&[])));
         assert_eq!(from_slice(b"()"), Ok(list(&[])));
         assert_eq!(from_slice(b"(())"), Ok(list(&[list(&[])])));
@@ -321,13 +319,7 @@ mod tests {
         assert_eq!(from_slice(b" (\t\"\")"), Ok(list(&[atom(b"")])));
         assert_eq!(from_slice_multi(b""), Ok(vec![]));
         assert_eq!(from_slice_multi(b"()"), Ok(vec![list(&[])]));
-        assert_eq!(
-            from_slice_multi(b"(\t\t\t)()"),
-            Ok(vec![list(&[]), list(&[])])
-        );
-        assert_eq!(
-            from_slice_multi(b"(\"\\\\\\n\")"),
-            Ok(vec![list(&[atom(b"\\\n")])])
-        );
+        assert_eq!(from_slice_multi(b"(\t\t\t)()"), Ok(vec![list(&[]), list(&[])]));
+        assert_eq!(from_slice_multi(b"(\"\\\\\\n\")"), Ok(vec![list(&[atom(b"\\\n")])]));
     }
 }
